#!/bin/bash

#### WG-Setup ################################################
# Aufruf: admin_setup
#
# Variablen:    HOMA_SETUP_URL
#               WG_SERVER_PUBKEY  WG_SERVER_IP  WG_SERVER_PORT
#               DEBUG

# Globale Variablen, die wir fuer das WG-Setup brauchen
declare -g TOKEN CLIENT_IP TAG PUBKEY PSK

WG_CONF_FILE="/etc/wireguard/wg0.conf"

declare -A ALLOWED_IPS=( \
[LUIS]="10.5.1.0/25, 10.5.1.128/25," \
[Chancenvielfalt]="130.75.153.0/28, 130.75.153.16/28, 130.75.153.48/29, 130.75.153.56/29," \
[FWU_WBBAU]="130.75.153.32/29, 130.75.153.40/29," \
[Geobot]="130.75.56.64/27, 130.75.56.96/27," \
[ITE]="130.75.92.192/28, 130.75.92.208/28," \
[LSE]="130.75.168.128/26, 130.75.168.192/26," \
[MaPhy]="130.75.75.192/27, 130.75.75.224/27," \
[Phoenixd]="130.75.195.128/26, 130.75.195.192/26," \
[PTB]="130.75.211.0/27, 130.75.211.32/27," \
[ZQS]="130.75.156.0/25, 130.75.156.128/25," \
)

## Umwandlung von base64 in base64url und zurueck
base64_to_url ()   { echo -n "$1" | tr '+/' '-_' | tr -d '=' ; }
base64_from_url () { echo -n "$1" | tr '-_' '+/' ; }

## Hole vom GARM-Server: TOKEN und CLIENT_IP
# Output: TOKEN CLIENT_IP
get_wg_ip () {
    $DEBUG && echo get_wg_ip || true
    #$ curl 'https://garm-dev.luis.uni-hannover.de/moin'
    #hwzklsnBekFSwFOqfckbybjbyCtwtNqh,10.10.10.2
    $DEBUG  && echo "curl -sS '$HOMA_SETUP_URL'"
    ANSWER="$( curl -sS "$HOMA_SETUP_URL" 2>"$STD_ERR_FILE" )"
    [ "$?" -gt "0" ] && error_msg "GARM: get_wg_ip" "Fehler beim Versuch eine Wireguard-IP zu erhalten:"  "$(<"$STD_ERR_FILE")" \
        && return 1 || true
    TOKEN="${ANSWER%,*}"
    CLIENT_IP="${ANSWER#*,}"
    $DEBUG && echo "TOKEN=$TOKEN CLIENT_IP=$CLIENT_IP" || true
    # Check if Answer really is a Token and a IP address
    if ! [ "$TOKEN,$CLIENT_IP" = "$ANSWER" \
        -a "$( echo $TOKEN | sed -e 's/[0-9a-zA-Z]*/TEST/' )" = "TEST" \
        -a "$( echo $CLIENT_IP | sed -e 's/[0-9]*\.[0-9]*\.[0-9]*\.[0-9]*/TEST/' )" = "TEST" ]; then
        error_msg "GARM: get_wg_ip" "Fehler beim Versuch eine Wireguard-IP zu erhalten: Antwort ist kein Token,IP-Paar."  "$(<"$STD_ERR_FILE")"
        return 2
    fi
}

## Read service tag or serial number from Bios or create a random tag
# Output:  TAG
get_tag () {
    $DEBUG && echo get_tag || true
    READ_TAG="$( sudo dmidecode -s system-serial-number )"
    [ "$?" -gt "0" ] && info_msg_timeout "Service-Tag nicht gefunden" "Fehler beim Versuch die Seriennummer / Service-Tag auszulesen." "Generiere ein zufaelliges Tag."
    READ_TAG="${READ_TAG:-$(dd if=/dev/urandom bs=6 count=1 2>/dev/null|base32)}"
    # Cleanup the Tag
    TAG="$( echo "$READ_TAG" | sed -e 's/[^0-9a-zA-Z_-]*//g' )"
    [ "$TAG" = "NotSpecified" ] && TAG="$(hostname)"
    $DEBUG && echo "READ_TAG='$READ_TAG' TAG='$TAG'" || true
    if [ "$READ_TAG" != "$TAG" ]; then
        info_msg_timeout "Service-Tag Encoding" "Die ausgelesene Seriennummer wurde angepasst." "Nutze als Tag: <b>$TAG</b>"
    fi
    TAG="lin_${TAG:0:7}"
}

## Wireguard-Konfiguration erzeugen
# Input:  CLIENT_IP  WG_SERVER_PUBKEY  WG_SERVER_IP  WG_SERVER_PORT
# Output: PUBKEY PSK
create_wg_config () {
    ! WG="$( which wg )" && error_msg "Wireguard Setup" "Konnte Wireguard-Binary (wg) nicht finden." \
        && return 20 || true
    $DEBUG && echo create_wg_config || true
    PRIVKEY="$( $WG genkey  2>"$STD_ERR_FILE" )"
    $DEBUG && echo "PRIVKEY=$PRIVKEY" && cat "$STD_ERR_FILE" || true
    [ "$?" -gt "0" ] && error_msg "Wireguard Setup" "Konnte Wireguard Private Key nicht erzeugen:" "$(<"$STD_ERR_FILE")" \
        && return 21 || true
    PUBKEY="$( $WG pubkey <<< "$PRIVKEY"  2>"$STD_ERR_FILE" )"
    $DEBUG && echo "PUBKEY=$PUBKEY" && cat "$STD_ERR_FILE" || true
    [ "$?" -gt "0" ] && error_msg "Wireguard Setup" "Konnte Wireguard PubKey nicht erzeugen:" "$(<"$STD_ERR_FILE")" \
        && return 22 || true
    PSK="$( $WG genpsk  2>"$STD_ERR_FILE" )"
    $DEBUG && echo "PSK=$PSK" && cat "$STD_ERR_FILE" || true
    [ "$?" -gt "0" ] && error_msg "Wireguard Setup" "Konnte Wireguard Pre Shared Key nicht erzeugen:" "$(<"$STD_ERR_FILE")" \
        && return 23 || true
    # FIXME evtl CIDR-Mask mit dazu bei Address?
    cat > "$WG_CONF_FILE" <<-EOF
[Interface]
PrivateKey = $PRIVKEY
Address = $CLIENT_IP
MTU = 1280
DNS = 130.75.1.32, 130.75.1.40
PostUp  = iptables -w -I OUTPUT ! -o %i -m mark ! --mark \$(wg show %i fwmark) -m addrtype ! --dst-type LOCAL -j REJECT
PreDown = iptables -w -D OUTPUT ! -o %i -m mark ! --mark \$(wg show %i fwmark) -m addrtype ! --dst-type LOCAL -j REJECT


[Peer]
PresharedKey = $PSK
Endpoint = $WG_SERVER_IP:$WG_SERVER_PORT
PublicKey = $WG_SERVER_PUBKEY
AllowedIPs = ${ALLOWED_IPS[@]} 0.0.0.0/0, ::/0
PersistentKeepalive = 25
EOF
    chmod go= "$WG_CONF_FILE"
    $DEBUG && cat "$WG_CONF_FILE" || true
}

# WG-Konfiguration zu GARM hochladen
# Input:  TOKEN CLIENT_IP TAG PUBKEY PSK
upload_wg_data () {
    $DEBUG && echo upload_wg_data || true
    # Upload-Daten zusammenstellen nach folgendem Schema; zusaetzlich mit Umbruechen fuer Info-Fenster.
    #$ curl 'https://garm-dev.luis.uni-hannover.de/moinmoin?ip=10.10.10.2&token=hwzklsnBekFSwFOqfckbybjbyCtwtNqh&tag=tagbla&pubkey=cHVia2V5YmxhCg&psk=pskbla'
    #ACK
    UPLOAD_DATA="moinmoin?ip=$CLIENT_IP&token=$TOKEN&tag="$TAG"&pubkey=$(base64_to_url "$PUBKEY")&psk=$(base64_to_url "$PSK")"
    HOMA_UPLOAD_URL="${HOMA_SETUP_URL%/*}/$UPLOAD_DATA"
    HOMA_UPLOAD_INFO="$( echo -e "$HOMA_UPLOAD_URL" | tr '?&' '\n\n' )"

    $DEBUG && echo "curl -sS '$HOMA_UPLOAD_URL'"  || true
    ANSWER="$( curl -sS "$HOMA_UPLOAD_URL" 2>"$STD_ERR_FILE" )"
    if [ "$?" -gt "0" -o "$ANSWER" != "ACK" ] ; then
    $DEBUG && echo "$ANSWER" && echo "$(<"$STD_ERR_FILE")" || true
        error_msg  "GARM-Upload" "Fehler beim Versuch die Daten zu GARM hochzuladen." \
"$HOMA_UPLOAD_INFO

Fehlermeldung:
$(<"$STD_ERR_FILE")
$ANSWER"
        return 5
    else
        NOW="$(date +%s)"
        info_msg "Wireguard-Aktivierung erfolgreich." "Wireguard-Aktivierung war erfolgreich." \
        "Der Client kann nun durch den Merge-Request\n\n<big><big><b>${TAG}_${CLIENT_IP}_${NOW:0:5}...</b></big></big>\n\nfreigeschaltet werden."
    fi
}

## Wireguard Service aktivieren
enable_wg_service () {
    $DEBUG && echo enable_wg_service || true
    # FIXME Fehler abfangen?
    sudo systemctl enable wg-quick@wg0.service
}

## Wireguard-Einrichtung
wg_setup () {
    $DEBUG && echo "wg_setup" || true
    check_done "wg_setup" && return 0
    get_wg_ip \
    && get_tag \
    && create_wg_config \
    && upload_wg_data \
    && enable_wg_service \
    && mark_done "wg_setup"
}

#### WG-Setup - Ende ###########################################
